/*
############################################################################## 
# LendItem.java
#
# Stellt ein LendItem Element dar.
# Hierbei werden die im LendItem enthaltenene Elemente (lentbyname,
# lentmymatno und lenddate) in einer Hashtable aufbewahrt, wobei der
# Hashschlssel der Elementname ist und der Hashwert der Elementwert.
#
# Beispiel LendItem:
# <lenditem>
# 	<lentbyname>A. Leiher</lentbyname>
#	<lentbymatno>1234567</lentbymatno>
#	<lenddate>01 04 2002</lenddate>
# </bibitem>
# Jonas Pohlandt 2002
##############################################################################
*/
import javax.servlet.http.HttpServletRequest;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.*;

import java.util.*;
import java.lang.Integer;

public class LendItem {

    // ## private Variablen ##

    private Hashtable elements = new Hashtable();

    // ## Konstruktoren ##

    // Leerer Konstruktor
    public LendItem() {}

    // Konstruiert ein LendItem aus einem HttpServletRequest.
    //
    // Parameter: Der Request aus dem das LendItem konstruiert werden soll.
    public LendItem(HttpServletRequest req) {

        Enumeration parameternames;
        String      parameter;
        String      value;
	
	// Element des LendItem mit den Werten der entsprechenden
	// Parameter des Requests fllen
        elements.put("key", req.getParameter("uniqueid"));
        // elements.put("lentbymatno", req.getParameter("lentbymatno"));
        elements.put("lentbyname", req.getParameter("lentbyname"));
        elements.put("lenddate", req.getParameter("lenddate"));
    }
    
    // Konstruiert ein LendItem aus einem Node, welcher selber ein
    // LendItem dsarstellen muss.
    //
    // Parameter: Der Node aus dem das LendItem konstruiert werden soll.
    public LendItem(Node node) {
	
	// Alle KinElemente des Knoten in eine NodeList schreiben
        NodeList childNodesList = node.getChildNodes();
	
	// Fr alle KindElemente:
        for (int j = 0; j < childNodesList.getLength(); j++) {
            Node childNode = childNodesList.item(j); //j-tes Kind des Bibitem

            if (childNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
		// Nur Elemente berprfen

                // Elemente auffllen
                NodeList elementChildsList = childNode.getChildNodes();

                for (int k = 0; k < elementChildsList.getLength(); k++) {
		    // Alle Kinder des Elementes prfen
                    Node cnode = elementChildsList.item(k);

                    if (cnode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
			// Nur der text interessiert
                        elements.put(childNode.getNodeName(),
                                     cnode.getNodeValue());
                    }
                }
            }
        }
    }

    // Methoden fr den direkten Zugriff auf die Hashtable
    public String getElement(String element) {
        String elementvalue = (String) elements.get(element);
        return elementvalue;
    }

    public void setElement(String element, String value) {
        elements.put(element, value);
    }

    // Diese Methode erzeugt einen Node, welcher mit dem LendItem
    // identisch ist
    //
    // Rckgabewert: Der erzeugte Node
    public Node toNode() {

        Node                   node;
        Element                element = null;
        Document               document;
        Enumeration            e;
     
     	// Um einen DOM Node zu erzeugen, bentigt man ein DOM
     	// Document welches die zur Node Erzeugung notwendigen
     	// Methoden zur Verfgung stellt.
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(true);

        try {
            DocumentBuilder builder = factory.newDocumentBuilder();

            document = builder.newDocument();
            element  = (Element) document.createElement("lenditem");
            e        = elements.keys();

	    // Fr alle Felder des LendItems (lentbyname, lentbymatno,
	    // lenddate):
            while (e.hasMoreElements()) {
                String  key   = (String) e.nextElement();
                // Element (Node) erzeugen.
                Element child = (Element) document.createElement(key);
		
		// In den am Element "hngenden" TextNode den
		// entsprechenden Wert schreiben.
                child.appendChild((Text) document
                    .createTextNode((String) elements.get(key)));
                element.appendChild(child);
            }
        
        // Mgliche Exceptions fangen    
        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
        }

	// Jedes Element ist auch ein Node
        node = (Node) element;
        return node;
    }

    // Diese Methode trennt einen String bei den Leerzeichen auf und
    // gibt ein Array zurck.
    // Bsp: String s = new String("a b c");
    // String[] sa = splitValues(s); -> String[1] ist jetzt "b"
    //
    // Parameter: Der zu splittende String
    //
    // Rckgabewert: Das Array mit den einzelnen Strings
    private String[] splitValues(String s) {

        int    blankCount = 0;
        int    i          = 0;
        String string     = s;

        // Zuerst wird gezhlt, wieviele Leerzeichen im String
        // enthalten sind.  Diese Information wird zum Anlegen eines
        // gengend grossen Array bentigt, welches spter die
        // einzelnen Strings enthalten soll.
        
        // Solange noch Leerzeichen im String enthalten sind:
        while (string.indexOf(" ") != -1) {
            blankCount++;

            // Erstes Leerzeichen und alle chars links davon entfernen
            string = string.substring(string.indexOf(" ") + 1);
        }

        // Neues Array mit entsprechender Gre anlegen
        String[] values = new String[blankCount + 1];

        // Array fllen
        while (s.indexOf(" ") != -1) {
            values[i] = s.substring(0, s.indexOf(" "));
            s         = s.substring(s.indexOf(" ") + 1);

            i++;
        }

        // Letzter String wird nicht mehr von der while Schleife
        // erfasst, deshalb manuell:
        values[i] = s;

        return values;
    }

    // Diese Methode sortiert eine Enumeration alphanumerisch
    //
    // Parameter: Die zu sortierende Enumaration
    //
    // Rckgabewert: die sortierte Enumeration
    private Enumeration sortParameters(Enumeration e) {

        Vector v = new Vector(0);

        while (e.hasMoreElements()) {
            v.addElement(e.nextElement());
        }

        java.util.Collections.sort(v);

        return v.elements();
    }

    // Diese Methode erkennt Metadaten.
    // Rckgabewert: true falls MetaDaten, sonst false    
    private boolean isMetaData(String s) {
        boolean isMetaData;
        isMetaData = s.matches(".*(bool|requester|button|uniqueid)");
        return isMetaData;	
    }
}
